
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% MLS.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [point,par,step,info]=MLS(fun,point,step,par,tune,info);
% try to significantly improve function value by Delta
%
% for details of input and output structures see VSBBO.m
%
function [point,par,step,info] = MLS(fun,point,step,par,tune,info)

% get point
point.xm    = point.X(:,point.b);
point.fm    = point.F(point.b);
point.xinit = point.xm; 
point.finit = point.fm; 
% get step
step.r      = 0; 
step.q      = zeros(point.n,1); 
% get par
par.good    = 0; 
par.state   = -1;
% get other parameters
t=0; info.ndf = 0; nE=0; nfDT = 0; stateTest = '';

if info.prt>=0, disp('start MLS'), end; 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%% main loop %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
while t<par.T

  switch par.state
  case -1 % new direction
    t=t+1;
    if par.ss, sub = (mod(t,tune.R)==0 & t<par.T); else sub = 0; end
    if t <= tune.H
      par.dir       = 1;   % pick heuristic direction
      stepType      = 'H'; % heuristic direction
      info.nstep.nH = info.nstep.nH+1; 
    elseif sub
      par.dir   = 2;   % pick subspace direction
      stepType  = 'S'; % subspace direction
      info.nstep.nS = info.nstep.nS+1;  
    elseif t<par.T
      par.dir   = 3;   % pick random direction
      stepType  = 'R'; % Random direction
      info.nstep.nR = info.nstep.nR+1;
    else
      if tune.cum==2 & step.r>=step.Delta % second cum step
      elseif tune.cum==1, step.q=point.x-point.xinit; % first cum step
      end
      if tune.cum>0 & all(step.q==0), par.dir=3; % pick random step
        stepType  = 'R'; % Random direction
        info.nstep.nR = info.nstep.nR+1; 
      else
        par.dir = 4; % pick cumulative step
        if tune.cum==1, par.state = 1; point.fl = point.finit; end 
                       % use initial value
        stepType  = 'C'; % cumulstive direction
        info.nstep.nC = info.nstep.nC+1; 
      end
    end
    [step,par] = direction(point,step,par,tune,info);
  case 0  % extrapolate at most T times
    nE = nE+1;
    if nE==1, point.fl = point.fm; else, step.p=2*step.p; end
    point.xm = point.X(:,point.b); point.fm = point.F(point.b);
    stepType  = 'E'; % extrapolate
    info.nstep.nE = info.nstep.nE+1;
  case 1 % opposite direction
    step.p = -step.p; point.fl = point.fr;
    stepType  = 'O'; % opposite direction
    info.nstep.nO = info.nstep.nO+1;
  end

  stateTest=[stateTest,stepType]; % update state
  % compute xr and fr
  point.xr = point.xm+ step.p; 

  % get and regularize new function value
  info.nf   = info.nf+1; 
  nfDT     = nfDT+1;
  xnew = max(point.low,min(point.upp,point.xr)); 
  fnew = fun(xnew);
  if isnan(fnew), fnew=inf; end;
  point.fr=max(-1e50,min(fnew,1e50));

     % check stopping test
     sec=(cputime-info.initTime);
     info.done=(sec>info.secmax)|(info.nf>=info.nfmax);
     info.done=info.done|(point.fr<=info.ftarget);
     if info.prt>0, MLSnf=info.nf, end;

     % assess quality of step
     df = point.fm-point.fr;
     if df>0 
         % best point improved
         info.ndf = info.ndf+1; 
         [point] = updateXF(point,tune); % update X and F
         if info.done, return; end;

         % check large gain for extrapolation
         ext  = (df>tune.gamma3*step.Delta);
         sext = (nE<tune.E & ext);
         if sext, par.state =0; continue; end
         
          % update lam and cumulative direction
          par.lam = max(par.lam,abs(point.fl+...
              point.fr-2*point.fm)/(step.dp)^2);
          if tune.cum ==2, [step] = updatecum(point,step,tune); end
         
         if ~ ext 
            point.xm = point.X(:,point.b); point.fm = point.F(point.b);
            if info.prt>=0,
               disp('df>0 but ext=0 ; xm and fm must be upadated');
            end
         end
        
     else   
         if info.done, return; end;

         % test for expected good opposite gain
         opp = (par.state<0&df<-step.Delta); 
         if opp, par.state =1; continue; end
         
        
         % update lam and cumulative direction
         lam1=abs(point.fl+point.fr-2*point.fm)/(step.dp)^2;
         par.lam = max(par.lam,lam1);
         if tune.cum ==2, [step] = updatecum(point,step,tune); end
         
     end
     
     par.state = -1; % line search completed
     
      % determine gain and good
      gain = point.finit-point.F(point.b);
      good = (gain>step.Delta);
      if good
         par.good = good;
         stateTest=[stateTest,'G'];
         break; 
      end
    
     % check upper bound for the number of function evaluations
     if nfDT>2*par.T+tune.E
        if info.prt>=0,
           stateTest 
           info.nstep 
           disp('nf > 2T+E in MLS.m')
        end
     end
     
end % of while t<par.T
info.nstuck = info.nstep.nE + info.nstep.nO+info.nstep.nC+info.nstep.nR;
info.stateTest = stateTest;
info.laststate =  stateTest(end);  %   state of last iteration
if info.prt>=0, stateTest, end

if info.prt>=0, disp('end MLS'), end; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
