
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% setScales.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [point,step,par,info]=setScales(fun,point,step,par,tune,info)
% estimate Deltamax and s
%
% for details of input and output structures see VSBBO.m
%
function [point,step,par,info] = setScales(fun,point,step,par,tune,info)
 
% get and regularize first function value
info.nf=1;
point.x = max(point.low,min(point.upp,point.x)); 
fnew = fun(point.x);
if isnan(fnew), fnew=inf; end;
point.f=max(-1e50,min(fnew,1e50));

% get initial values for subspace 
point.F = point.f; point.X = point.x; 
point.xm = point.x; point.fm = point.f; 
point.b = 1; point.m =1;

% get initial values for T
par.T = tune.H+(tune.S-1)*(tune.R+1)+2; par.ss = 0; par.lam = 0; 

% get initial values for sdelta and Delta
step.deltamin = tune.deltainit;
step.deltamax = tune.deltainit;
step.delta = tune.deltainit;  
step.Delta = step.Deltamax;   

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%             get X in order to estimate Deltamax                % 
%             MLS doesn't use subspace direction                 %  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                               
for t=1:tune.T0                                                  
   [point,par,step,info] = MLS(fun,point,step,par,tune,info);
   if info.done, return; end;
   if info.prt>=0, nfSS=info.nf, end;
   if point.m == 1, [point] = updateXF(point,tune);  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                get maximal desied gain Deltamax                %      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


par.ss = 1;

for i=1:point.m
    dX(:,i)  = point.X(:,i)-point.X(:,point.b);
    normX(i) = norm(dX(:,i));
end

s = max(dX')'; step.s(s==0) = 1;

dF = median(abs(point.F-point.F(point.b))); 

if dF==0
   mdX = median(normX); 
   if mdX~=0
      step.Deltamax = (tune.gamma2)*sqrt(mdX);
      step.Delta    = step.Deltamax;
      par.lam       = (tune.gamma4)*sqrt(mdX/point.n);
   else
       if step.Deltamax==0
          par.lam = 1/sqrt(point.n);
       else
           par.lam = step.Deltamax/sqrt(point.n);
       end
   end
else
   step.Deltamax = tune.gamma2*dF;
   step.Delta    = step.Deltamax;
   par.lam       = (tune.gamma4)*dF/sqrt(point.n);
  
end


 step.deltamin = tune.gamma6*par.hss;
 step.deltamax = tune.gamma7*par.hss;
 step.delta    = step.deltamax;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
